// HRulerView.m
// Project: Rulers
//
// Stephan Wacker
// 93-02-11


#import "HRulerView.h"
#import "HRulerControl.h"
#import "RulerPreferencesControl.h"
#import "math.h"			// ceil()


@implementation HRulerView


- initFrame: (const NXRect *) frameRect
// Initialize yourself after instance has been created.
{
    NXSize	offset;
    
    [super initFrame: frameRect];
    
    // Move origin (from left bottom corner) to the right
    // by 20% of frame width (truncated to an integer).
    offset.width = floor( 0.2 * NX_WIDTH( frameRect ) );
    offset.height = 0.0;
    [self moveOrigin: &offset];
    
    return self;
}



- drawSelf: (const NXRect *) rects : (int) rectCount
// Draw the marks in the ruler image.
// Background is drawn by super.
{
    NXCoord		x;
    int			k;
    char		str[40];
    NXCoord		tick_baselength, tick_length;
    const TickMark	*mark;
    BOOL		displayLabels = [control labelsMode];
    int			marksPosition = [control hMarksMode];
    int			posNumDirection = [control hPosNumMode];
    NXFontMetrics	*metrics = [markFont metrics];	// font dimensions
    float		capHeight = metrics->capHeight * [markFont pointSize];
    
    [super drawSelf: rects : rectCount];
    
    NXSetColor( [control marksColor] );
    PSsetlinewidth( 0.25 );			// thin lines
    tick_baselength = 18.0;			// main mark length
    [markFont set];
    
    // Process every set of marks.
    //
    for( mark = [self tickMarkSystem]; mark->distance > 0.0; mark++ )
    {
	tick_length = mark->length * tick_baselength;
	
	// start with
	//  x = k * mark->distance + origin.x,
	//  x >= NX_X(&bounds)
	for(
	  k = (int) ceil( (NX_X(&bounds)-origin.x)/mark->distance ),
	   x = k * mark->distance + origin.x,
	   k *= mark->value;
	  x <= NX_MAXX(&bounds);
	  k += mark->value, x += mark->distance )
	{
	    if( marksPosition & HMARKS_BOTTOM ) {
		PSmoveto( x, NX_Y(&bounds) );
		PSrlineto( 0.0, tick_length );	// a tick mark
	    }
	    if( marksPosition & HMARKS_TOP ) {
		PSmoveto( x, NX_MAXY(&bounds) );
		PSrlineto( 0.0, -tick_length );	// a tick mark
	    }
    
	    if( displayLabels ) {
		if( mark->value > 0 )
		{
		    sprintf( str, "%d",
		      k < 0 ? ((posNumDirection & POSNUM_LEFT ) ? -k : k)
		            : ((posNumDirection & POSNUM_RIGHT) ? k : -k)
		    );
		    PSmoveto( x - 0.5 * ([markFont getWidthOf: str]-1.0),
		      0.5 * (NX_HEIGHT(&bounds) - (capHeight + 0.0)) );
		    PSshow( str );
		}
		else if( mark->value < 0 )
		{
		    // display a fraction like `3/8'
		    // !toDo!
		}
	    }
	}
    }
    
    PSstroke();					// draw all tick marks
    
    return self;
}



- (NXCoord) constrainDistance: (NXCoord) distance
                        frame: (NXRect *) frameRect
// PRIVATE
// Change the frameRect's width by distance.
// Observe minimum and maximum sizes.
// Answer actual width change.
{
    NXSize	minSize, maxSize;
    NXCoord	minDist, maxDist;
    NXCoord	result = distance;
    
    [window getMinSize: &minSize];
    [window getMaxSize: &maxSize];
    
    maxDist = maxSize.width - NX_WIDTH( frameRect );
    minDist = minSize.width - NX_WIDTH( frameRect );
    
    if( result < minDist ) result = minDist;
    if( result > maxDist ) result = maxDist;
    
    if( result * distance < 0.0 ) result = 0.0;	// cancel opposite direction
    
    NX_WIDTH( frameRect ) += result;
    
    return result;
}



- sizeRight: (const NXSize *) delta
// Change the window's width by delta (to the right).
// Positive distance increases width.
{
    NXCoord	distance = delta->width;
    
    if( !(distance = [self constrainDistance: distance frame: &rubberRect]) )
	return nil;				// size is not changed
    
    [self drawRubberband];
    
    return self;
}



- sizeLeft: (const NXSize *) delta
// Change the window's width by delta (to the left).
// Positive distance decreases width.
{
    NXCoord	distance = -delta->width;
    
    if( !(distance = [self constrainDistance: distance frame: &rubberRect]) )
	return nil;				// size is not changed
    
    NX_X( &rubberRect ) -= distance;		// move left edge
    origin.x += distance;			// adjust local origin
    
    [self drawRubberband];
    
    return self;
}


@end
