// RulerView.m
// Project: Rulers
//
// Stephan Wacker
// 93-02-01


#import "RulerView.h"
#import "RulerControl.h"
#import "RulerPreferences.h"	// We only need the constants, not the methods

#import "Rubber.h" // psw stuff

#define RUBBERBAND_WINDOW_WIDTH	1
#define CONTINUOUS_UPDATE	1 // YES/NO doesn't work in #ifdef



// Tick mark systems.

TickMark TMpt[] = {			// 1 point is one pixel on the screen
    100.0,	1.0,	100,	// 100 pt
    50.0,	0.7,	0,	//  50 pt
    10.0,	0.3,	0,	//  10 pt
    5.0,	0.1,	0,	//   5 pt
    0.0,	0.0,	0
};

TickMark TMpc[] = {			// 1 pica = 12pt
    48.0,	1.0,	4,	// 4 pc
    24.0,	0.6,	0,	// 2 pc
    12.0,	0.3,	0,	// 1 pc
    4.0,	0.1,	0,	// 4 pt
    0.0,	0.0,	0
};

TickMark TMin[] = {			// 1 inch = 72pt
    72.0,	1.0,	1,	// 1 in
    72.0/2.0,	0.6,	-2,	// 1/2 in
    72.0/4.0,	0.3,	0,	// 1/4 in
    72.0/8.0,	0.1,	0,	// 1/8 in
    0.0,	0.0,	0
};

TickMark TMin_75[] = {			// 75 dpi
    75.0,	1.0,	1,	// 1 in
    75.0/2.0,	0.6,	-2,	// 1/2 in
    75.0/4.0,	0.3,	0,	// 1/4 in
    75.0/8.0,	0.1,	0,	// 1/8 in
    0.0,	0.0,	0
};

TickMark TMin_92[] = {			// 92 dpi
    92.0,	1.0,	1,	// 1 in
    92.0/2.0,	0.6,	-2,	// 1/2 in
    92.0/4.0,	0.3,	0,	// 1/4 in
    92.0/8.0,	0.1,	0,	// 1/8 in
    0.0,	0.0,	0
};

TickMark TMin_10[] = {			// 1/10 inch marks
    72.0,	1.0,	1,	// 1 in
    72.0*0.5,	0.7,	0,	// 0.5 in
    72.0*0.1,	0.3,	0,	// 0.1 in
    0.0,	0.0,	0
};

TickMark TMin_75_10[] = {		// 75 dpi
    75.0,	1.0,	1,	// 1 in
    75.0*0.5,	0.7,	0,	// 0.5 in
    75.0*0.1,	0.3,	0,	// 0.1 in
    0.0,	0.0,	0
};

TickMark TMin_92_10[] = {		// 92 dpi
    92.0,	1.0,	1,	// 1 in
    92.0*0.5,	0.7,	0,	// 0.5 in
    92.0*0.1,	0.3,	0,	// 0.1 in
    0.0,	0.0,	0
};

#define ONE_CM		(72.0 / 2.54)
TickMark TMcm[] = {			// 2.54 centimeter = 1 inch
    ONE_CM,	1.0,	1,	// 1 cm
    ONE_CM*0.5,	0.7,	0,	// 0.5 cm
    ONE_CM*0.1,	0.3,	0,	// 0.1 cm
    0.0,	0.0,	0
};

#define ONE_CM_75		(75.0 / 2.54)
TickMark TMcm_75[] = {			// 75 dpi
    ONE_CM_75,		1.0,	1,	// 1 cm
    ONE_CM_75*0.5,	0.7,	0,	// 0.5 cm
    ONE_CM_75*0.1,	0.3,	0,	// 0.1 cm
    0.0,		0.0,	0
};

#define ONE_CM_92		(92.0 / 2.54)
TickMark TMcm_92[] = {			// 92 dpi
    ONE_CM_92,		1.0,	1,	// 1 cm
    ONE_CM_92*0.5,	0.7,	0,	// 0.5 cm
    ONE_CM_92*0.1,	0.3,	0,	// 0.1 cm
    0.0,		0.0,	0
};



@implementation RulerView


- initFrame: (const NXRect *) frameRect
// Initialize yourself after instance has been created.
{
    const char *pbTypes[] = { NXColorPboardType };

    [super initFrame: frameRect];
    
    origin.x = 0.0, origin.y = 0.0;		// left bottom corner
    [self synchScreenOrigin];
    
    markFont
     = [Font newFont: "Helvetica-Bold"
		size: 14
	      matrix: NX_IDENTITYMATRIX
       ];

    [self registerForDraggedTypes: pbTypes count: 1];
    
    return self;
}



- drawSelf: (const NXRect *) rects : (int) rectCount
// Draw the ruler image's background.
// Should be refined by subclasses.
{
    NXSetColor( [control color] );
    
    switch( rectCount ) {
      case 1 :
	NXRectFill( rects );		// one exposed region
      break;
      case 3 :
	NXRectFill( rects+1 );		// two exposed regions
	NXRectFill( rects+2 );
      break;
      default :
	NXRectFill( &bounds );		// complete view
      break;
    }
    
    return self;
}



- (BOOL) acceptsFirstMouse
// Accept mouse even if not key window.
{
    return YES;
}



- mouseDown: (NXEvent *) event
// Drag the local origin with the mouse cursor.
{
    [self trackMouseFor: self action: @selector( moveOrigin: )];
    [window display];
    
    return self;
}



- rightMouseDown: (NXEvent *) event
// Do nothing and wait for rightMouseUp.
{
#if CONTINUOUS_UPDATE
    NXPoint	loc = event->location;		// window coordinates
    
    [self convertPoint: &loc fromView: nil];	// view coordinates
    [self setOrigin: &loc];
    [self trackMouseFor: self action: @selector( moveOrigin: )];
    [window display];
#endif

    return self;
}



- rightMouseUp: (NXEvent *) event
// Set the local origin to the mouse cursor.
{
    NXPoint	loc = event->location;		// window coordinates
    
    [self convertPoint: &loc fromView: nil];	// view coordinates
    [self setOrigin: &loc];
    [window display];
    
    return self;
}



- getOrigin : (NXPoint *) point
// Get coordinate system's origin (relative to the view).
{
    *point = origin;
    return self;
    
    return self;
}



- setOrigin: (const NXPoint *) point
// Set local coordinate system's origin.
// Caller should send a display message to the view or its window.
{
    origin = *point;
    [self synchScreenOrigin];
#if CONTINUOUS_UPDATE
    [self update];
#else
    [self setNeedsDisplay: YES];
#endif
    
    return self;
}



- moveOrigin: (const NXSize *) delta
// Move local coordinate system's origin.
// Caller should send a display message to the view or its window.
{
    origin.x += delta->width, origin.y +=delta->height;
    [self synchScreenOrigin];
#if CONTINUOUS_UPDATE
    [self update];
#else
    [self setNeedsDisplay: YES];
#endif
    
    return self;
}



- synchOrigin
// Take origin from screenOrigin.
{
    origin = screenOrigin;
    [window convertScreenToBase: &origin];
    [self setNeedsDisplay: YES];
    
    return self;
}



- synchScreenOrigin
// Take screenOrigin from origin.
{
    screenOrigin = origin;
    [window convertBaseToScreen: &screenOrigin];
    
    return self;
}



- (const TickMark *) tickMarkSystem
// Answer current tick mark system, as selected by the RulerControl.
{
    switch( [control unitsMode] ) {
    
      case UNIT_PT :
	return TMpt;
      break;
      
      case UNIT_PC :
      default :
	return TMpc;
      break;
      
      case UNIT_IN :
	switch( [control inchMode] ) {
	  case INCH_DIV_8 :
	  default :
	    return TMin;
	  case INCH_DIV_10 :
	    return TMin_10;
	}
      break;
      
      case UNIT_CM :
	return TMcm;
      break;
      
      case UNIT_TRUE_IN :
	switch( [control screenMode] ) {
	  case XVII_INCH :
	  default :
	    switch( [control inchMode] ) {
	      case INCH_DIV_8 :
	      default :
		return TMin_92;
	      case INCH_DIV_10 :
		return TMin_92_10;
	    }
	  case XXI_INCH :
	    switch( [control inchMode] ) {
	      case INCH_DIV_8 :
	      default :
		return TMin_75;
	      case INCH_DIV_10 :
		return TMin_75_10;
	    }
	}
      break;
      
      case UNIT_TRUE_CM :
	switch( [control screenMode] ) {
	  case XVII_INCH :
	  default :
	    return TMcm_92;
	  case XXI_INCH :
	    return TMcm_75;
	}
      break;
    }
    
    return NULL;
}



- trackMouseFor: target action: (SEL) action
//
// Track the mouse while it drags some control object.
// Initiated by a mouseDown: event; tracks mouse until mouseUp:.
//
// The action is repeatedly sent to the target with
// the partial moving distance as its argument.
// The method definition should be
//    action: (NXSize *) delta
//
{
    NXEvent	*event = [NXApp currentEvent];		// MouseDown
    NXPoint	oldLoc = event->location;		// window coordinates
    NXPoint	newLoc;
    NXSize	delta;
    int		oldMask = [window eventMask];
    int		trackMask;
    int		stopEvent;

    switch( event->type )
    {
      default:
      case NX_LMOUSEDOWN:
	trackMask = NX_LMOUSEUPMASK | NX_LMOUSEDRAGGEDMASK;
	stopEvent = NX_LMOUSEUP;
	break;
      case NX_RMOUSEDOWN:
	trackMask = NX_RMOUSEUPMASK | NX_RMOUSEDRAGGEDMASK;
	stopEvent = NX_RMOUSEUP;
	break;
    }
    
    [window setEventMask: (oldMask | trackMask)];
    
    [window convertBaseToScreen: &oldLoc];		// screen coordinates
    
    while( 1 ) {
	newLoc = event->location;			// window coordinates
	[window convertBaseToScreen: &newLoc];		// screen coordinates
	
#if 0
 This was an ugly mistake: If the cursor was moved in both x and y direction, the first test signaled that delta.width was not zero and the second test -- including the assignment to delta.height -- was skipped.
	if( (delta.width = newLoc.x - oldLoc.x) != 0.0
	 || (delta.height = newLoc.y - oldLoc.y) != 0.0
	) {
	    [target perform: action with: (id) &delta];
	}
#else
	delta.width = newLoc.x - oldLoc.x;
	delta.height = newLoc.y - oldLoc.y;
	if( delta.width || delta.height ) {
	    [target perform: action with: (id) &delta];
	}
#endif
	
	if( event->type == stopEvent ) break;		// from while()
	
	oldLoc = newLoc;
	event = [NXApp getNextEvent: trackMask];
    }
    
    [window setEventMask: oldMask];		// restore old settings
    
    return self;
}


- showRubberband
{
    PSWInitRubberband();
    [window getFrame: &rubberRect];
    [self drawRubberband];
    PSWMakeVisible(TRUE);
    return self;
}


- hideRubberband
{
    PSWMakeVisible(FALSE);
    [window placeWindowAndDisplay: &rubberRect];
    return self;
}


// The code for this method is copied from
//	/BGCD/BGCDDeveloper/Sources/Classes/Rubberview/Rubberband.m
//	by Slugg Jello, 
//	Mouthing Flowers
//	152 20th Ave. #1
//	Seattle, WA. 98112
//	slugg@mouthers.nwnexus.wa.com
//
- drawRubberband
{
    NXRect	tRect, screenRect = rubberRect;
    
    struct
    {
	NXRect	top, left, bottom, right;
    } points;
    
    
    /**** set size of each rubberband side ****/
    
    /* left */
    tRect = screenRect;
    tRect.size.width = RUBBERBAND_WINDOW_WIDTH;
    tRect.size.height = MAX(RUBBERBAND_WINDOW_WIDTH,screenRect.size.height);
    points.left = tRect;
    
    /* right */
    tRect.origin.x += screenRect.size.width - 1.0;
    points.right = tRect;
    
    /* bottom */
    tRect = screenRect;
    tRect.origin.x += 1.0;
    tRect.size.width = MAX(RUBBERBAND_WINDOW_WIDTH,screenRect.size.width-1.0);
    tRect.size.height = RUBBERBAND_WINDOW_WIDTH;
    points.bottom = tRect;
    
    /* top */
    tRect.origin.y += screenRect.size.height - 1.0;
    points.top = tRect;
    
    /* this'll erase old one and draw new one in new position */
    PSWPositionRect( (float *)&points );

    return self;
}


@end // RulerView



@implementation RulerView ( DraggingDestination )


- (NXDragOperation) draggingEntered: sender
// Invoked when the dragged image enters the destination.
// Answer what we will do when the object is released.
{
    return ([sender draggingSourceOperationMask] & NX_DragOperationGeneric);
}


- (BOOL) prepareForDragOperation: sender
// Invoked when the image is released.
// Answer whether we will perform the drag operation.
{
    return YES;
}


- (BOOL) performDragOperation: sender
// Invoked after the released image has been removed from the screen.
// Answer whether the data has been accepted.
{
    Pasteboard	*pb = [Pasteboard newName: NXDragPboard];

    [window disableDisplay];
    [self setNeedsDisplay: YES];
    [control setColor: NXReadColorFromPasteboard(pb)];
    [[window reenableDisplay] display];
    
    return YES;
}


@end //RulerView ( DraggingDestination )
